# Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 1999-2007 Peter Thoeny, peter@thoeny.org
# and TWiki Contributors. All Rights Reserved. TWiki Contributors
# are listed in the AUTHORS file in the root of this distribution.
# NOTE: Please extend that file, not this notice.
#
# This module is based/inspired on Catalyst framework. Refer to
#
# http://search.cpan.org/~mramberg/Catalyst-Runtime-5.7010/lib/Catalyst.pm
# 
# for credits and liscence details.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# As per the GPL, removal of this notice is prohibited.

=begin twiki

---+!! package TWiki::Engine::CGI

Class that implements default CGI behavior.

Refer to TWiki::Engine documentation for explanation about methos below.

=cut

package TWiki::Engine::CGI;

use strict;
use base 'TWiki::Engine';
use TWiki::Request;
use TWiki::Request::Upload;
use TWiki::Response;

sub run { shift; TWiki::UI::handleRequest(@_) }

sub prepareRequest {
    my ($this, $req, %args) = @_;
    delete $this->{env};
    $this->{env} = $args{env} if $args{env};
}

sub prepareConnection {
    my ( $this, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;

    $req->remoteAddress( $ENV{REMOTE_ADDR} );
    $req->remoteUser( $ENV{REMOTE_USER} );
    $req->method( $ENV{REQUEST_METHOD} );

    if ( $ENV{HTTPS} && uc( $ENV{HTTPS} ) eq 'ON' ) {
        $req->secure(1);
    }

    if ( $ENV{SERVER_PORT} && $ENV{SERVER_PORT} == 443 ) {
        $req->secure(1);
    }
}

sub prepareQueryParameters {
    my ( $this, $req ) = @_;
    local(*ENV) = $this->{env} || \%ENV;
    
    $this->SUPER::prepareQueryParameters($req, $ENV{QUERY_STRING})
        if $ENV{QUERY_STRING};
}

sub prepareHeaders {
    my ( $this, $req ) = @_;
    local(*ENV) = $this->{env} || \%ENV;

    foreach my $header ( keys %ENV ) {
        next unless $header =~ /^(?:HTTP|CONTENT|COOKIE)/i;
        ( my $field = $header ) =~ s/^HTTPS?_//;
        $req->header( $field => $ENV{$header} );
    }
}

sub preparePath {
    my ( $this, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;

    # SMELL: "The Microsoft Internet Information Server is broken with
    # respect to additional path information. If you use the Perl DLL
    # library, the IIS server will attempt to execute the additional
    # path information as a Perl script. If you use the ordinary file
    # associations mapping, the path information will be present in the
    # environment, but incorrect. The best thing to do is to avoid using
    # additional path information."
    
    # Clean up PATH_INFO problems, e.g.  Support.CobaltRaqInstall.  A valid
    # PATH_INFO is '/Main/WebHome', i.e. the text after the script name;
    # invalid PATH_INFO is often a full path starting with '/cgi-bin/...'.
    my $pathInfo = $ENV{PATH_INFO} || '';
    my $cgiScriptName = $ENV{SCRIPT_NAME} || '';
    $pathInfo =~ s{$cgiScriptName/}{/}i;
    $req->pathInfo( $pathInfo );
    $req->uri( $ENV{REQUEST_URI} );
}

sub prepareBody {
    my ( $this, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;
   
    return unless $ENV{CONTENT_LENGTH};
    CGI::initialize_globals();
    my $cgi = new CGI();
    my $err = $cgi->cgi_error;
    throw TWiki::EngineException($1, $2)
        if defined $err && $err =~ /\s*(\d{3})\s*(.*)/o;
    $this->{cgi} = $cgi;
}

sub prepareBodyParameters {
    my ( $this, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;
    
    return unless $ENV{CONTENT_LENGTH};
    my %p = $this->{cgi}->Vars;
    while ( my ( $key, $value ) = each %p ) {
        $req->bodyParam( -name => $key, -value => $value );
        $this->{uploads}->{$key} = 1 if scalar $this->{cgi}->upload($key);
    }
}

sub prepareUploads {
    my ( $this, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;

    return unless $ENV{CONTENT_LENGTH};
    my %uploads;
    foreach my $key ( keys %{ $this->{uploads} } ) {
        my $fname = $this->{cgi}->param($key);
        $uploads{$key} = new TWiki::Request::Upload(
            name    => $key,
            headers => $this->{cgi}->uploadInfo($fname),
            tmpname => $this->{cgi}->tmpFileName($fname),
        );
    }
    delete $this->{uploads};
    $req->uploads( \%uploads );
}

sub finalizeUploads {
    my ( $this, $res, $req ) = @_;
    local (*ENV) = $this->{env} || \%ENV;

    $req->delete($_) foreach keys %{ $req->uploads };
    undef $_[0]->{cgi};
}

sub finalizeHeaders {
    my ( $this, $res, $req ) = @_;
    
    $this->SUPER::finalizeHeaders( $res, $req );
    $this->finalizeCookies($res);
    foreach my $header ( keys %{ $res->headers } ) {
        print $header . ': ' . $_ . $this->CRLF
          foreach $res->getHeader($header);
    }
    print $this->CRLF;
}

sub write {
    my ( $this, $buffer ) = @_;
    print $buffer;
}

1;
