# Module of TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 1999-2007 Peter Thoeny, peter@thoeny.org
# and TWiki Contributors. All Rights Reserved. TWiki Contributors
# are listed in the AUTHORS file in the root of this distribution.
# NOTE: Please extend that file, not this notice.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# As per the GPL, removal of this notice is prohibited.

=begin twiki

---+!! package TWiki::Engine::HTTP::Restarter

Class that implements an HTTP stand alone server capable of
dealing with twiki and static requests. This engine also restarts 
itself whenever lib file changes are detected.

Refer to TWiki::Engine documentation for explanation about methos below.

=cut

package TWiki::Engine::HTTP::Restarter;

use strict;
use base 'TWiki::Engine::HTTP';
use HTTP::Request;
use HTTP::Headers;
use Time::HiRes 'sleep';
use IO::Socket::INET;
use File::Monitor;
use File::Spec;
use File::Find;

sub run {
    my $this = shift;

    my $pid = fork;
    die "Can't fork: $!\n" unless defined $pid;
    my %args = (
        port => 3000,
        host => 0,
        @_
    );
    unless ( $pid ) {
        close STDIN;
        close STDOUT;
        my $monitor = _setupMonitor();
        my $serverIp = $this->_canonicalIP($args{host});
        $serverIp = '127.0.0.1' if $serverIp eq '0.0.0.0';
        while (1) {
            my @changed = grep { $_->is_mtime } $monitor->scan;
            if ( @changed ) {
                my $client = new IO::Socket::INET(
                        PeerAddr => $serverIp,
                        PeerPort => $args{port},
                )
                or die "Can't create client socket (is server running?): ",
                $!;

                my $kill = new HTTP::Request('RESTART', '/', 
                    new HTTP::Headers(Connection => 'close') );
                $kill->protocol('HTTP/1.0');
                $client->send( $kill->as_string )
                    or die "Can't send restart instruction: ", $!;
                $client->close();
                last;
            }
            sleep $args{rdelay};
        }
        exit(0);
    }
    $this->SUPER::run(@_);
}

sub _setupMonitor {
    my $monitor = new File::Monitor();
    finddepth(
        {
            wanted => sub {
                $monitor->watch(
                    { name => $File::Find::name, recurse => 0, files => 0 } )
                    if !-l $File::Find::name &&
                        (
                          (
                            -f $File::Find::name &&
                            $File::Find::name =~ /\.(?:pm|cfg|spec)$/
                          ) || 
                          -d $File::Find::name
                        );
            },
            follow   => 0,
            no_chdir => 1,
        },
        join( '', ( File::Spec->splitpath( $INC{'TWiki.pm'} ) )[ 0, 1 ] ),
    );
    $monitor->watch( (caller(1))[1] );
    $monitor->watch($_) foreach values %INC;
    $monitor->scan();
    return $monitor;
}

1;
