#!/usr/bin/perl -wT
#
# TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 1999-2007 Peter Thoeny, peter@thoeny.org
# and TWiki Contributors.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# As per the GPL, removal of this notice is prohibited.

use strict;
use warnings;

BEGIN {
    @INC = ( '.', grep { $_ ne '.' } @INC );
    require 'setlib.cfg';
    $TWiki::cfg{Engine} = 'TWiki::Engine::HTTP';
}

use Getopt::Long;
use Pod::Usage;

my ( $pubUrl, $scriptUrl, $detach, $host, $port, $restart, $rdelay, $help);

BEGIN {
    my @argBackup = @ARGV;
    GetOptions(
        'puburl=s'    => \$pubUrl,
        'scripturl=s' => \$scriptUrl,
        'detach|d'    => \$detach,
        'host|h=s'    => \$host,
        'port|p=i'    => \$port,
        'restart|r'   => \$restart,
        'rdelay=f'    => \$rdelay,
        'help|?'      => \$help,
    );
    $rdelay ||= 1;
    $TWiki::cfg{Engine} .= '::Restarter' if $restart;
    @ARGV = @argBackup;
}

pod2usage(1) if $help;

use TWiki;
use TWiki::UI;

use Carp;
local $SIG{__DIE__} = \&Carp::confess unless $detach;

my %args = ();

$args{pubUrl}    = $pubUrl    if defined $pubUrl;
$args{scriptUrl} = $scriptUrl if defined $scriptUrl;
$args{detach}    = $detach    if defined $detach;
$args{host} = TWiki::Sandbox::untaintUnchecked($host) if defined $host;
$args{port} = $port if defined $port;

$TWiki::engine->run(%args, rdelay => $rdelay);

__END__

=head1 SYNOPSIS

twiki_http.pl [options]

  Options:
       --puburl     Url path to pub. Defaults to $TWiki::cfg{PubUrlPath}
       --scripturl  Url path to bin. Defaults to $TWiki::cfg{ScriptUrlPath}
    -d --detach     Detach from terminal and run in background. Defaults to false.
    -p --port       Port (defaults to 3000)
    -h --host       Host (defaults to all)
    -r --restart    Restart server when files gets modified. Defaults to false.
       --rdelay     Delay between check for changes
    -? --help       Display this help and exits

=cut
